package more.crawler.nmc.Utils;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zuo
 * @date 2022/9/15 16:08
 */
@Slf4j
public class HttpHysplitUtils {

    /**
     * 向指定URL发送GET方法的请求
     *
     * @param url   发送请求的URL
     * @param param 请求参数，请求参数应该是 name1=value1&name2=value2 的形式。
     * @return URL 所代表远程资源的响应结果
     */
    public static Map<String, Object> sendGet(String url, String param, String cookie, boolean isRetransmission) {
        Map<String, Object> returnMap = new HashMap<>();
        StringBuilder html = new StringBuilder();
        BufferedReader in = null;
        try {
            String urlNameString = url + (StringUtils.isBlank(param) ? "" : "?" + param);
            URL realUrl = new URL(urlNameString);
            // 打开和URL之间的连接
            URLConnection connection = realUrl.openConnection();
            // 设置通用的请求属性
            connection.setRequestProperty("Accept", "*/*");
            connection.setRequestProperty("Connection", "keep-alive");
            connection.setRequestProperty("User-Agent",
                    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36");
            connection.setRequestProperty("Host", "www.nmc.cn");
            if (StringUtils.isNotBlank(cookie)) {
                connection.setRequestProperty("Cookie", cookie);
            }
            connection.setConnectTimeout(15000);
            connection.setReadTimeout(20000);
            // 建立实际的连接
            connection.connect();
            // 获取所有响应头字段
            Map<String, List<String>> headerFields = connection.getHeaderFields();
            returnMap.put("header", headerFields);
            // 定义 BufferedReader输入流来读取URL的响应
            in = new BufferedReader(new InputStreamReader(
                    connection.getInputStream())); //这里如果出现乱码，请使用带编码的InputStreamReader构造方法，将需要的编码设置进去
            String line;
            while ((line = in.readLine()) != null) {
                html.append(line);
            }
        } catch (Exception e) {
            if (isRetransmission){
                try {
                    Thread.sleep(500);
                    return sendGet(url, param, cookie, false);
                } catch (Exception ex) {
                    log.error("url:" + url + "连接失败，错误为：" + e);
                }
            }else {
                log.error("url:" + url + "连接失败，错误为：" + e);
            }
        }
        // 使用finally块来关闭输入流
        finally {
            try {
                if (in != null) {
                    in.close();
                }
            } catch (Exception e2) {
                e2.printStackTrace();
            }
        }
        returnMap.put("html", html.toString());
        return returnMap;
    }

    public static void getData(String html) {
        //获取的数据，存放在到Bid的对象中，自己可以重新建立一个对象存储
//        Bid bid = new Bid();
        //采用Jsoup解析
        Document doc = Jsoup.parse(html);
        // System.out.println("doc内容" + doc.text());
        //获取html标签中的内容tr
        Elements elements = doc.select("option");
        System.out.println(elements.size() + "****条");
        //循环遍历数据
        for (Element element : elements) {
            if (element.select("td").first() == null) {
                continue;
            }
            Elements tdes = element.select("td");
            for (int i = 0; i < tdes.size(); i++) {
                Elements select = tdes.get(i).select("option");
                if (select.select("option").first() == null) {
                    continue;
                }
                System.out.println(select);
//                this.relation(tdes,tdes.get(i).text(),bid,i+1);
            }
        }
//        return bid;
    }
}
