package more.crawler.nmc.Utils;

import org.apache.commons.io.IOUtils;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.springframework.lang.Nullable;
import sun.misc.BASE64Encoder;

import javax.net.ssl.*;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Objects;

/**
 * @author zuo
 * @date 2022/11/30 18:57
 */

public class HttpRequest {


    /**
     * get请求并下载文件
     *
     * @param url
     * @param path 文件路径及名称
     * @return 是否成功
     * @throws
     */
    public static boolean getFileDown(String url, String path) {
        try {
            URLConnection connection = new URL(url).openConnection();
            connection.setConnectTimeout(5000);
            connection.setReadTimeout(180000);
            connection.connect();
            return processInFileDown(connection, path);
        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }
    }

    private static boolean processInFileDown(URLConnection connection, String path) {
        InputStream is = null;
        OutputStream os = null;
        try {
            is = connection.getInputStream();
            os = new FileOutputStream(path);
            byte[] buffer = new byte[1024];
            int bytesRead;
            //读取到缓冲区
            while ((bytesRead = is.read(buffer)) != -1) {
                os.write(buffer, 0, bytesRead);
            }
            is.close();
            //刷新OutputStream，将缓冲的数据写入文件
            os.flush();
            os.close();
            return true;
        } catch (IOException e) {
            e.printStackTrace();
            return false;
//            throw new HttpException(-1);
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (os != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * 通过URL获取图片并转成Base64格式
     *
     * @param imgUrl
     * @return
     */
    public static String urlToBase64(String imgUrl) {
        InputStream inputStream = null;
        ByteArrayOutputStream outputStream = null;
        byte[] buffer = null;
        try {
            // 创建URL
            URL url = new URL(imgUrl);
            // 创建链接
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            conn.setRequestMethod("GET");
            conn.setConnectTimeout(5000);
            inputStream = conn.getInputStream();
            outputStream = new ByteArrayOutputStream();
            // 将内容读取内存中
            buffer = new byte[1024 * 1024];
            int len = -1;
            while ((len = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, len);
            }
            buffer = outputStream.toByteArray();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (inputStream != null) {
                try {
                    // 关闭inputStream流
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (outputStream != null) {
                try {
                    // 关闭outputStream流
                    outputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        // 对字节数组Base64编码
        String base64 = "data:image/png;base64," + new BASE64Encoder().encode(buffer);
        return base64;
    }

    public static String Post(String httpUrl, @Nullable String param, @Nullable String httpToken) {
        StringBuffer result = new StringBuffer();
        HttpsURLConnection connection = null;
        OutputStream os = null;
        InputStream is = null;
        BufferedReader br = null;
        try {
            HttpsURLConnection.setDefaultHostnameVerifier(new HttpRequest().new NullHostNameVerifier());
            SSLContext sc = SSLContext.getInstance("TLS");
            sc.init(null, trustAllCerts, new SecureRandom());
            HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
            URL url = new URL(httpUrl);
            connection = (HttpsURLConnection) url.openConnection();
            connection.setRequestMethod("POST");
            connection.setConnectTimeout(15000);
            connection.setReadTimeout(15000);
            connection.setDoOutput(true);
            connection.setDoInput(true);
            //如果不设置setFixedLengthStreamingMode(0),即使设置了Content-Length 服务端也是返回411
            if (null == param || param.equals("")) {
                connection.setFixedLengthStreamingMode(0);
            }
            connection.setRequestProperty("Content-Type", "application/json");
            connection.setRequestProperty("Content-Length", null != param && !param.equals("") ? String.valueOf(param.getBytes().length) : "0");
            if (null != httpToken && !httpToken.equals("")) {
                connection.setRequestProperty("Authorization", "Bearer " + httpToken);
            }

            if (null != param && !param.equals("")) {
                os = connection.getOutputStream();
                os.write(param.getBytes());
            }
            if (connection.getResponseCode() == 200) {
                is = connection.getInputStream();
                if (null != is) {
                    br = new BufferedReader(new InputStreamReader(is));
                    String temp = null;
                    while (null != (temp = br.readLine())) {
                        result.append(temp);
                    }
                }
            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        } catch (KeyManagementException e) {
            throw new RuntimeException(e);
        } finally {
            if (br != null) {
                try {
                    br.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (os != null) {
                try {
                    os.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            connection.disconnect();
        }
        return result.toString();
    }

    static TrustManager[] trustAllCerts = new TrustManager[]{new X509TrustManager() {
        @Override
        public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
            // TODO Auto-generated method stub
        }

        @Override
        public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
            // TODO Auto-generated method stub
        }

        @Override
        public X509Certificate[] getAcceptedIssuers() {
            // TODO Auto-generated method stub
            return null;
        }
    }};

    public class NullHostNameVerifier implements HostnameVerifier {
        /*
         * (non-Javadoc)
         *
         * @see javax.net.ssl.HostnameVerifier#verify(java.lang.String,
         * javax.net.ssl.SSLSession)
         */
        @Override
        public boolean verify(String arg0, SSLSession arg1) {
            // TODO Auto-generated method stub
            return true;
        }
    }

    public static void uploadImage(String imageUrl, String serverUrl, String sign) {
        InputStream in = null;
        try (CloseableHttpClient client = HttpClients.createDefault()) {
            HttpPost post = new HttpPost(serverUrl);
            if (!StringUtils.isEmpty(sign)) {
                post.setHeader("sign", sign);
            }
            MultipartEntityBuilder builder = MultipartEntityBuilder.create();
            in = new URL(imageUrl).openStream();
            File file = stream2file(in);
            builder.addBinaryBody("input", file, ContentType.create("image/jpeg"), file.getName());
//            // 添加参数
//            builder.addTextBody("type", "7");
//            builder.addTextBody("dataTime", "2024/09/27 20:00:00");

            post.setEntity(builder.build());

            HttpResponse response = client.execute(post);
            System.out.println("Response Code: " + response.getStatusLine().getStatusCode());

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (!Objects.isNull(in)) {
                    in.close();
                }
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
    }

    static final String PREFIX = "stream2file";//前缀字符串定义文件名；必须至少三个字符
    static final String SUFFIX = ".jpeg";//后缀字符串定义文件的扩展名；如果为null，则将使用后缀".tmp"

    public static File stream2file(InputStream in) throws IOException {
        final File tempFile = File.createTempFile(PREFIX, SUFFIX);
        tempFile.deleteOnExit();
        try (FileOutputStream out = new FileOutputStream(tempFile)) {
            IOUtils.copy(in, out);
            out.close();
            in.close();
        }
        return tempFile;
    }
}
