package more.crawler.nmc.configuration;

import com.alibaba.fastjson.JSON;
import more.crawler.nmc.Utils.DesCipherUtil;
import more.crawler.nmc.response.ResponseResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Calendar;
import java.util.Date;
import java.util.Random;

/**
 * 内部传输拦截校验
 * @author zuo
 * @date 2022/12/9 17:39
 */

@Component
@CrossOrigin
public class TransmissionInterceptor implements HandlerInterceptor {

    @Autowired
    private NmcConfig nmcConfig;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        String key = nmcConfig.getSignKey();
        String userId = nmcConfig.getSignUserId();


        /**
         * 添加sign(添加响应头)
         */
        Random random = new Random();

        String responseSign = String.format("%03d", random.nextInt(999)) +  (new Date().getTime() /1000) +
                String.format("%03d", random.nextInt(999)) + userId + String.format("%03d", random.nextInt(999));
        response.addHeader("sign", DesCipherUtil.encrypt(responseSign,key));


        /**
         * 校验sign（获取响应头）
         */
        String requestSign = request.getHeader("sign");
        if (requestSign!=null) {
            try {
                String encrypt = DesCipherUtil.decrypt(requestSign, key);
//                String encrypt = DesCipherUtil.decrypt(sign, "D5A574A9");


                Long time = Long.valueOf(encrypt.substring(3, 13));
                Calendar ca = Calendar.getInstance();
                ca.add(Calendar.SECOND,-300);
                long minTime = ca.getTime().getTime() / 1000;
                ca.add(Calendar.SECOND,600);
                long maxTime = ca.getTime().getTime() / 1000;
                if (minTime > time || time > maxTime){
                    doResponse(response,"sign already expired！");
                    return false;
                }

                if (!encrypt.contains("918170978046845723")){
                    doResponse(response,"Invalid sign！");
                    return false;
                }
                return true;
            } catch (Exception e){
                doResponse(response,"Invalid sign！");
                return false;
            }
        }
        doResponse(response,"sign already expired！");
        return false;
    }

    public void doResponse(HttpServletResponse response,String info) throws IOException {
        response.setContentType("application/json");
        response.setCharacterEncoding("utf-8");
        PrintWriter writer = response.getWriter();
//        ResponseEntity<String> responseEntity=new ResponseEntity<>(info, HttpStatus.NOT_ACCEPTABLE);
        String json = JSON.toJSONString(ResponseResult.error(info));
        writer.write(json);
        writer.flush();
        writer.close();
    }
}
