package more.crawler.nmc.quartz.service;

import lombok.extern.slf4j.Slf4j;
import more.crawler.nmc.quartz.model.QuartzJob;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.List;


/**
 * @Description: 定时任务在线管理
 * @Author: jeecg-boot
 * @Date: 2019-04-28
 * @Version: V1.1
 */
@Slf4j
@Service
public class QuartzJobService {

    @Autowired
    private Scheduler scheduler;

    /**
     * 项目启动时，初始化定时器
     * 主要是防止手动修改数据库导致未同步到定时任务处理（注：不能手动修改数据库ID和任务组名，否则会导致脏数据）
     */
    public void init() {

        List<QuartzJob> quartzJobs = new ArrayList<>();
        QuartzJob nmcJob = new QuartzJob();
        nmcJob.setId("nmc-001");
        nmcJob.setDelFlag(0);
        nmcJob.setJobClassName("more.crawler.nmc.quartz.job.NmcJob");
        nmcJob.setCronExpression("0 15 0/2 * * ? *");
//        nmcJob.setCronExpression("* * * * * ? *");

        nmcJob.setDescription("中央气象台数据下载");
        nmcJob.setStatus(1);
        quartzJobs.add(nmcJob);
        for (QuartzJob quartzJob : quartzJobs) {
            this.schedulerAdd(quartzJob.getId(), quartzJob.getJobClassName().trim(), quartzJob.getCronExpression().trim(), quartzJob.getParameter());
            System.out.println("运行脚本(id:" + quartzJob.getId() + ",内容:" + quartzJob.getDescription() + ")");
        }
    }

    private static Job getClass(String classname) throws Exception {
        Class<?> class1 = Class.forName(classname);
        return (Job) class1.newInstance();
    }

    /**
     * 添加定时任务
     *
     * @param jobClassName
     * @param cronExpression
     * @param parameter
     */
    private void schedulerAdd(String id, String jobClassName, String cronExpression, String parameter) {
        try {
            // 启动调度器
            scheduler.start();

            // 构建job信息
            JobDetail jobDetail = JobBuilder.newJob(getClass(jobClassName).getClass()).withIdentity(id).usingJobData("parameter", parameter).build();

            // 表达式调度构建器(即任务执行的时间)
            CronScheduleBuilder scheduleBuilder = CronScheduleBuilder.cronSchedule(cronExpression);

            // 按新的cronExpression表达式构建一个新的trigger
            CronTrigger trigger = TriggerBuilder.newTrigger().withIdentity(id).withSchedule(scheduleBuilder).build();

            scheduler.scheduleJob(jobDetail, trigger);
        } catch (SchedulerException e) {
            throw new RuntimeException("创建定时任务失败");
        } catch (RuntimeException e) {
            throw new RuntimeException(e.getMessage());
        } catch (Exception e) {
            throw new RuntimeException("后台找不到该类名：" + jobClassName);
        }
    }
}
