package more.crawler.nmc.response;

import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.ToString;
import lombok.experimental.Accessors;

import java.io.Serializable;

/**
 * @author YeJR
 * @version 1.0
 * @date 2020/2/19 13:11
 * 前端返回对象
 */
@Data
@ToString
@NoArgsConstructor
@Accessors(chain = true)
public class ResponseResult<T> implements Serializable {

    /**
     * 返回码
     */
    private int code;

    /**
     * 信息
     */
    private String msg;

    /**
     * 数据，任意类型
     */
    private T data;


    public ResponseResult(ResponseCode responseCode) {
        this.code = responseCode.getCode();
        this.msg = responseCode.getMsg();
    }


    /**
     * 成功
     * @param data
     * @param <T>
     * @return
     */
    public static <T> ResponseResult<T> success(T data){
        ResponseResult<T> responseResult = new ResponseResult<>(ResponseCode.SUCCESS);
        responseResult.setData(data);
        return responseResult;
    }

    /**
     * 失败
     * @param msg
     * @param <T>
     * @return
     */
    public static <T> ResponseResult<T> error(String msg){
        ResponseResult<T> responseResult = new ResponseResult<>(ResponseCode.ERROR);
        responseResult.setMsg(msg);
        return responseResult;
    }

    /**
     * 失败
     * @param code
     * @param msg
     * @param <T>
     * @return
     */
    public static <T> ResponseResult<T> error(int code, String msg){
        ResponseResult<T> responseResult = new ResponseResult<>();
        responseResult.setMsg(msg).setCode(code);
        return responseResult;
    }

}
